@@ -484,30 +484,6 @@ extend_list(bitarrayobject *self, PyObject *list)
     return 0;
 }
 
-static int
-extend_tuple(bitarrayobject *self, PyObject *tuple)
-{
-    PyObject *item;
-    Py_ssize_t n, i;
-
-    assert(PyTuple_Check(tuple));
-    n = PyTuple_GET_SIZE(tuple);
-    if (n == 0)
-        return 0;
-
-    if (resize(self, self->nbits + n) < 0)
-        return -1;
-
-    for (i = 0; i < n; i++) {
-        item = PyTuple_GET_ITEM(tuple, i);
-        if (item == NULL)
-            return -1;
-        if (set_item(self, self->nbits - n + i, item) < 0)
-            return -1;
-    }
-    return 0;
-}
-
 static int
 extend_01(bitarrayobject *self, PyObject *bytes)
 {
@@ -531,7 +507,7 @@ extend_01(bitarrayobject *self, PyObject *bytes)
         case '1': vi = 1; break;
         default:
             PyErr_Format(PyExc_ValueError,
-                         "character must be '0' or '1', found '%c'", c);
+                         "character must be '0' or '1', got '%c'", c);
             return -1;
         }
         setbit(self, self->nbits - nbytes + i, vi);
@@ -552,9 +528,6 @@ extend_dispatch(bitarrayobject *self, PyObject *obj)
     if (PyList_Check(obj))                                    /* list */
         return extend_list(self, obj);
 
-    if (PyTuple_Check(obj))                                  /* tuple */
-        return extend_tuple(self, obj);
-
     if (PyBytes_Check(obj)) {                             /* bytes 01 */
 #ifdef IS_PY3K
         PyErr_SetString(PyExc_TypeError,
@@ -588,6 +561,7 @@ extend_dispatch(bitarrayobject *self, PyObject *obj)
         Py_DECREF(iter);
         return res;
     }
+
     PyErr_Format(PyExc_TypeError,
                  "'%s' object is not iterable", Py_TYPE(obj)->tp_name);
     return -1;
@@ -647,12 +621,12 @@ IntBool_AsInt(PyObject *v)
         x = PyLong_AsLong(v);
     }
     else {
-        PyErr_SetString(PyExc_TypeError, "integer or bool expected");
+        PyErr_SetString(PyExc_TypeError, "int or bool expected");
         return -1;
     }
 
     if (x < 0 || x > 1) {
-        PyErr_SetString(PyExc_ValueError, "integer 0 or 1 expected");
+        PyErr_SetString(PyExc_ValueError, "int 0 or 1 expected");
         return -1;
     }
     return (int) x;
@@ -1032,7 +1006,7 @@ bitarray_invert(bitarrayobject *self, PyObject *args)
 }
 
 PyDoc_STRVAR(invert_doc,
-"invert(index=<all bits>)\n\
+"invert(index=<all bits>, /)\n\
 \n\
 Invert all bits in the array (in-place).\n\
 When the optional `index` is given, only invert the single bit at index.");
@@ -1264,7 +1238,7 @@ PyDoc_STRVAR(fromfile_doc,
 \n\
 Extend bitarray with up to n bytes read from the file object f.\n\
 When n is omitted or negative, reads all data until EOF.\n\
-When n is provided and positions but exceeds the data available,\n\
+When n is provided and positive but exceeds the data available,\n\
 EOFError is raised (but the available data is still read and appended.");
 
 
@@ -1668,7 +1642,10 @@ bitarray_subscr(bitarrayobject *self, PyObject *item)
         }
         return res;
     }
-    PyErr_SetString(PyExc_TypeError, "index or slice expected");
+
+    PyErr_Format(PyExc_TypeError,
+                 "bitarray indices must be integers or slices, not %s",
+                 Py_TYPE(item)->tp_name);
     return NULL;
 }
 
@@ -1820,11 +1797,16 @@ bitarray_ass_subscr(bitarrayobject *self, PyObject* item, PyObject* value)
         if (IS_INT_OR_BOOL(value))
             return setslice_bool(self, item, value);
 
-        PyErr_SetString(PyExc_IndexError,
-                        "bitarray or bool expected for slice assignment");
+        PyErr_Format(PyExc_TypeError,
+                     "bitarray or bool expected for slice assignment, not %s",
+                     Py_TYPE(value)->tp_name);
         return -1;
     }
-    return -1;                  /* cannot happen */
+
+    PyErr_Format(PyExc_TypeError,
+                 "bitarray indices must be integers or slices, not %s",
+                 Py_TYPE(item)->tp_name);
+    return -1;
 }
 
 static PyMappingMethods bitarray_as_mapping = {
@@ -1990,7 +1972,8 @@ static int
 check_codedict(PyObject *codedict)
 {
     if (!PyDict_Check(codedict)) {
-        PyErr_SetString(PyExc_TypeError, "dict expected");
+        PyErr_Format(PyExc_TypeError, "dict expected, got %s",
+                     Py_TYPE(codedict)->tp_name);
         return -1;
     }
     if (PyDict_Size(codedict) == 0) {
@@ -2028,7 +2011,8 @@ bitarray_encode(bitarrayobject *self, PyObject *args)
 
     iter = PyObject_GetIter(iterable);
     if (iter == NULL) {
-        PyErr_SetString(PyExc_TypeError, "iterable object expected");
+        PyErr_Format(PyExc_TypeError, "'%s' object is not iterable",
+                     Py_TYPE(iterable)->tp_name);
         return NULL;
     }
     /* extend self with the bitarrays from codedict */
@@ -2048,6 +2032,7 @@ bitarray_encode(bitarrayobject *self, PyObject *args)
     if (PyErr_Occurred())
         return NULL;
     Py_RETURN_NONE;
+
  error:
     Py_DECREF(iter);
     return NULL;
@@ -2455,9 +2440,6 @@ typedef struct {
 
 static PyTypeObject DecodeIter_Type;
 
-#define DecodeIter_Check(op)  PyObject_TypeCheck(op, &DecodeIter_Type)
-
-
 /* create a new initialized bitarray decode iterator object */
 static PyObject *
 bitarray_iterdecode(bitarrayobject *self, PyObject *obj)
@@ -2506,7 +2488,6 @@ decodeiter_next(decodeiterobject *it)
 {
     PyObject *symbol;
 
-    assert(DecodeIter_Check(it));
     symbol = binode_traverse(it->tree, it->bao, &(it->index));
     if (symbol == NULL)  /* stop iteration OR error occured */
         return NULL;
@@ -2582,8 +2563,6 @@ typedef struct {
 
 static PyTypeObject SearchIter_Type;
 
-#define SearchIter_Check(op)  PyObject_TypeCheck(op, &SearchIter_Type)
-
 /* create a new initialized bitarray search iterator object */
 static PyObject *
 bitarray_itersearch(bitarrayobject *self, PyObject *x)
@@ -2625,7 +2604,6 @@ searchiter_next(searchiterobject *it)
 {
     Py_ssize_t p;
 
-    assert(SearchIter_Check(it));
     p = search(it->bao, it->xa, it->p);
     if (p < 0)  /* no more positions -- stop iteration */
         return NULL;
@@ -2816,10 +2794,9 @@ bitarray_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
     if (initial == NULL || initial == Py_None)
         return newbitarrayobject(type, 0, endian);
 
-    /* boolean */
+    /* bool */
     if (PyBool_Check(initial)) {
-        PyErr_SetString(PyExc_TypeError,
-                        "cannot create bitarray from boolean");
+        PyErr_SetString(PyExc_TypeError, "cannot create bitarray from bool");
         return NULL;
     }
 
@@ -2968,8 +2945,6 @@ typedef struct {
 
 static PyTypeObject BitarrayIter_Type;
 
-#define BitarrayIter_Check(op)  PyObject_TypeCheck(op, &BitarrayIter_Type)
-
 /* create a new initialized bitarray iterator object, this object is
    returned when calling iter(a) */
 static PyObject *
@@ -2977,7 +2952,6 @@ bitarray_iter(bitarrayobject *self)
 {
     bitarrayiterobject *it;
 
-    assert(bitarray_Check(self));
     it = PyObject_GC_New(bitarrayiterobject, &BitarrayIter_Type);
     if (it == NULL)
         return NULL;
@@ -2994,7 +2968,6 @@ bitarrayiter_next(bitarrayiterobject *it)
 {
     long vi;
 
-    assert(BitarrayIter_Check(it));
     if (it->index < it->bao->nbits) {
         vi = GETBIT(it->bao, it->index);
         it->index++;
