@@ -7,18 +7,15 @@ import os
 import sys
 import unittest
 from string import hexdigits
-from random import choice, randint
-try:
-    from collections import Counter
-except ImportError:
-    pass
+from random import choice, randint, random
+from collections import Counter
 
-from bitarray import (bitarray, frozenbitarray, bits2bytes,
+from bitarray import (bitarray, frozenbitarray, bits2bytes, decodetree,
                       get_default_endian, _set_default_endian)
 from bitarray.test_bitarray import Util
 
-from bitarray.util import (zeros, make_endian, rindex, strip, count_n,
-                           count_and, count_or, count_xor, subset,
+from bitarray.util import (zeros, urandom, make_endian, rindex, strip,
+                           count_n, count_and, count_or, count_xor, subset,
                            ba2hex, hex2ba, ba2int, int2ba, huffman_code)
 
 if sys.version_info[0] == 3:
@@ -57,13 +54,54 @@ class TestsZeros(unittest.TestCase):
         self.assertRaises(TypeError, zeros, 1.0)
         self.assertRaises(ValueError, zeros, -1)
 
-        self.assertRaises(TypeError, zeros, 0, 1) # endian not string
-        self.assertRaises(ValueError, zeros, 0, 'foo') # endian wrong string
+        # endian not string
+        for x in 0, 1, {}, [], False, True:
+            self.assertRaises(TypeError, zeros, 0, x)
+        # endian wrong string
+        self.assertRaises(ValueError, zeros, 0, 'foo')
 
 tests.append(TestsZeros)
 
 # ---------------------------------------------------------------------------
 
+class TestsRandom(unittest.TestCase):
+
+    def test_1(self):
+        for default_endian in 'big', 'little':
+            _set_default_endian(default_endian)
+            a = urandom(0)
+            self.assertEqual(a, bitarray())
+            self.assertEqual(a.endian(), default_endian)
+
+            b  = urandom(0, endian=None)
+            self.assertEqual(b.endian(), default_endian)
+
+            for n in range(100):
+                a = urandom(n)
+                self.assertEqual(len(a), n)
+                self.assertEqual(b.endian(), default_endian)
+
+            a = urandom(1000)
+            b = urandom(1000)
+            self.assertNotEqual(a, b)
+            self.assertTrue(400 < a.count() < 600)
+            self.assertTrue(400 < b.count() < 600)
+
+    def test_wrong_args(self):
+        self.assertRaises(TypeError, urandom)
+        self.assertRaises(TypeError, urandom, '')
+        self.assertRaises(TypeError, urandom, bitarray())
+        self.assertRaises(TypeError, urandom, [])
+        self.assertRaises(TypeError, urandom, 1.0)
+        self.assertRaises(ValueError, urandom, -1)
+
+        self.assertRaises(TypeError, urandom, 0, 1)
+        self.assertRaises(ValueError, urandom, 0, 'foo')
+
+tests.append(TestsRandom)
+
+# ---------------------------------------------------------------------------
+
 class TestsMakeEndian(unittest.TestCase, Util):
 
     def test_simple(self):
@@ -219,9 +257,19 @@ tests.append(TestsStrip)
 
 class TestsCount_N(unittest.TestCase, Util):
 
+    @staticmethod
+    def count_n(a, n):
+        "return the index i for which a[:i].count() == n"
+        i, j = n, a.count(1, 0, n)
+        while j < n:
+            if a[i]:
+                j += 1
+            i += 1
+        return i
+
     def check_result(self, a, n, i):
         self.assertEqual(a.count(1, 0, i), n)
-        if i:
+        if i > 0:
             self.assertTrue(a[i - 1])
 
     def test_simple(self):
@@ -244,6 +292,7 @@ class TestsCount_N(unittest.TestCase, Util):
             i = count_n(a, n)
             self.check_result(a, n, i)
             self.assertEqual(a[:i].count(), n)
+            self.assertEqual(i, self.count_n(a, n))
         self.assertEQUAL(a, b)
 
     def test_frozen(self):
@@ -263,7 +312,7 @@ class TestsCount_N(unittest.TestCase, Util):
         for N in list(range(100)) + [1000, 10000, 100000]:
             a = bitarray(N)
             v = randint(0, 1)
-            a.setall(v - 1)
+            a.setall(not v)
             for _ in range(randint(0, min(N, 100))):
                 a[randint(0, N - 1)] = v
             n = randint(0, a.count())
@@ -350,14 +399,10 @@ class TestsBitwiseCount(unittest.TestCase, Util):
         self.assertEqual(count_or(a, b), 5)
         self.assertEqual(count_xor(a, b), 3)
 
-    def test_bit_count2(self):
+    def test_bit_count_random(self):
         for n in list(range(50)) + [randint(1000, 2000)]:
-            a = bitarray()
-            a.frombytes(os.urandom(bits2bytes(n)))
-            del a[n:]
-            b = bitarray()
-            b.frombytes(os.urandom(bits2bytes(n)))
-            del b[n:]
+            a = urandom(n)
+            b = urandom(n)
             self.assertEqual(count_and(a, b), (a & b).count())
             self.assertEqual(count_or(a, b),  (a | b).count())
             self.assertEqual(count_xor(a, b), (a ^ b).count())
@@ -680,7 +725,7 @@ class TestsIntegerization(unittest.TestCase, Util):
             if a.endian == 'big':
                 self.assertTrue(len(a) == 1 or a.index(1) == 0)
             self.assertEqual(ba2int(a), i)
-            if i > 0 and sys.version_info[:2] >= (2, 7):
+            if i > 0:
                 self.assertEqual(i.bit_length(), len(a))
             # add a few trailing / leading zeros to bitarray
             if endian == 'big':
@@ -754,6 +799,13 @@ class TestsHuffman(unittest.TestCase):
         self.assertRaises(TypeError, huffman_code, {'A': 'a', 'B': 1})
         self.assertRaises(ValueError, huffman_code, {})
 
+    def check_tree(self, code):
+        n = len(code)
+        tree = decodetree(code)
+        self.assertEqual(tree.todict(), code)
+        # ensure tree has 2n-1 nodes (n symbol nodes and n-1 internal nodes)
+        self.assertEqual(tree.nodes(), 2 * n - 1)
+
     def test_balanced(self):
         n = 6
         freq = {}
@@ -762,6 +814,7 @@ class TestsHuffman(unittest.TestCase):
         code = huffman_code(freq)
         self.assertEqual(len(code), 2 ** n)
         self.assertTrue(all(len(v) == n for v in code.values()))
+        self.check_tree(code)
 
     def test_unbalanced(self):
         N = 27
@@ -772,22 +825,29 @@ class TestsHuffman(unittest.TestCase):
         self.assertEqual(len(code), N)
         for i in range(N):
             self.assertEqual(len(code[i]), N - (1 if i <= 1 else i))
+        self.check_tree(code)
 
-    if sys.version_info[:2] >= (2, 7):
-        def test_counter(self):
-            message = 'the quick brown fox jumps over the lazy dog.'
-            code = huffman_code(Counter(message))
-            a = bitarray()
-            a.encode(code, message)
-            self.assertEqual(''.join(a.decode(code)), message)
+    def test_counter(self):
+        message = 'the quick brown fox jumps over the lazy dog.'
+        code = huffman_code(Counter(message))
+        a = bitarray()
+        a.encode(code, message)
+        self.assertEqual(''.join(a.decode(code)), message)
+        self.check_tree(code)
 
-        def test_rand_list(self):
-            plain = [randint(0, 100) for _ in range(500)]
-            code = huffman_code(Counter(plain))
-            a = bitarray()
-            a.encode(code, plain)
-            self.assertEqual(a.decode(code), plain)
+    def test_random_list(self):
+        plain = [randint(0, 100) for _ in range(500)]
+        code = huffman_code(Counter(plain))
+        a = bitarray()
+        a.encode(code, plain)
+        self.assertEqual(a.decode(code), plain)
+        self.check_tree(code)
 
+    def test_random_freq(self):
+        N = randint(2, 1000)
+        # create Huffman code for N symbols
+        code = huffman_code({i: random() for i in range(N)})
+        self.check_tree(code)
 
 tests.append(TestsHuffman)
 
