@@ -37,7 +37,7 @@ Create a bitarray of length, with all values 0, and optional
 endianness, which may be 'big', 'little'.
 """
     if not isinstance(length, (int, long) if _is_py2 else int):
-        raise TypeError("integer expected")
+        raise TypeError("int expected")
 
     a = bitarray(length, get_default_endian() if endian is None else endian)
     a.setall(0)
@@ -90,7 +90,7 @@ Allowed values for mode are the strings: `left`, `right`, `both`
     if not isinstance(a, bitarray):
         raise TypeError("bitarray expected")
     if not isinstance(mode, str):
-        raise TypeError("string expected for mode")
+        raise TypeError("str expected for mode")
     if mode not in ('left', 'right', 'both'):
         raise ValueError("allowed values 'left', 'right', 'both', got: %r" %
                          mode)
@@ -140,7 +140,7 @@ Bitarray of hexadecimal representation.
 hexstr may contain any number of hex digits (upper or lower case).
 """
     if not isinstance(s, (str, unicode if _is_py2 else bytes)):
-        raise TypeError("string expected, got: %r" % s)
+        raise TypeError("str expected, got: %r" % s)
 
     strlen = len(s)
     if strlen % 2:
@@ -205,10 +205,10 @@ If signed is False and a negative integer is given, an OverflowError
 is raised.
 """
     if not isinstance(i, (int, long) if _is_py2 else int):
-        raise TypeError("integer expected")
+        raise TypeError("int expected")
     if length is not None:
         if not isinstance(length, int):
-            raise TypeError("integer expected for length")
+            raise TypeError("int expected for length")
         if length <= 0:
             raise ValueError("integer larger than 0 expected for length")
     if signed and length is None:
